#include "..\shared\SubMission.h"

#include <string.h>

/*
	InstantMission

	Just add water!!
*/

#define MAX_GOODIES 40
// #define TEST_INSTANT 1

const int myForce = 1;
const int compForce = 2;
const int difficulty = 2;

enum AIPType {
	AIPType0=0,
	AIPType1,
	AIPType2,
	AIPType3,
	AIPTypeA,
	AIPTypeL,
	AIPTypeS,
	MAX_AIP_TYPE,
};
const char *AIPTypeExtensions="0123als"; // AIPType above is offset in this string table

char CustomAIPNameBase[256];
bool CheckedSVar3=false;
bool UseCustomAIPs;

class instantMission : public SubMission {
public:
	instantMission(void)
	{
		b_count = &b_last - &b_first - 1;
		b_array = &b_first + 1;

		f_count = &f_last - &f_first - 1;
		f_array = &f_first + 1;

		h_count = &h_last - &h_first - 1;
		h_array = &h_first + 1;

		i_count = &i_last - &i_first - 1;
		i_array = &i_first + 1;
	}
//	~instantMission();

	// Based on player races and type, set AIPlan
	void SetCPUAIPlan(const int CPUSide,const int HumanSide,AIPType Type);

	// And set up extra race-specific vehicles at pathpoints in the map.
	void SetupExtraVehicles(void);

	virtual void AddObject(Handle h);
	virtual void DeleteObject(Handle h);
	virtual void Setup(void);
	virtual void CreateObjectives();
	virtual void TestObjectives();
	virtual void Execute(bool *TeamIsSetUp, Handle *RecyclerHandles);
	virtual void DoGenericStrategy(bool *TeamIsSetUp, Handle *RecyclerHandles);
	virtual EjectKillRetCodes PlayerEjected(void);
	virtual EjectKillRetCodes PlayerKilled(int KillersHandle);

private:
	// bools
	bool
		b_first,
		start_done,
		inspect_bunker,
		second_objective,
		picking_up[MAX_GOODIES],
		captured[MAX_GOODIES],
		siege_on,
		anti_assault,
		late_game,
		have_armory,
		b_last;

	// floats
	float
		f_first,
		f,
		f_last;

	// handles
	Handle
		h_first,
		h,
		obj1,
		bunker,
		bunker2,
		player,
		foe1,
		foe2,
		foe3,
		walker,
		tug,
		recycler,
		enemy_recycler,
		powerup1,
		powerup2,
		powerup3,
		powerup4,
		goodies[MAX_GOODIES],
		tuglist[MAX_GOODIES],  // keep track of tugs
		holding[MAX_GOODIES],
		h_last;

	// integers
	int
		i_first,
		strat_team,
		i,
		siege_counter,
		assault_counter,
		next_wave,
		next_spawn,
		time_count,
		post_bunker,
		enemy_reinforcement_time,
		friend_reinforcement_time,
		end_counter,
		comp_team,
		powerup_counter,
		turn_counter,
		myGoal,
		mySide,
		HumanSide,
		NumHumans,
		difficulty,
		i_last;
};

// And set up extra race-specific vehicles at pathpoints in the map.
void instantMission::SetupExtraVehicles(void)
{
	int pathCount;
	char **pathNames;
	GetAiPaths(pathCount, pathNames);
	unsigned long j,k;

	for (int i = 0; i < pathCount; ++i) {
		char *Label = pathNames[i];
		if(strncmp(Label,"mpi",3)==0) {
			// Starts with MPI. Process it.

			const int MaxODFLen=32;
			char ScionStuff[MaxODFLen+1];
			char ISDFStuff[MaxODFLen+1];

			// Prezap full string contents so we don't have to null-term later
			memset(ScionStuff,0,sizeof(ScionStuff));
			memset(ISDFStuff,0,sizeof(ISDFStuff));

			j=0;

			// Skip forward until an _
			while((Label[j] != '_') && (Label[j] != '\0'))
				j++;
			if(Label[j]=='\0')
				continue; // Misformat! No _ found! Bail!

			k=0;
			j++; // skip the _. Now, copy out scion stuff
			while((Label[j] != '_') && (Label[j] != '\0')) {
				if(k<MaxODFLen) {
					ScionStuff[k]=Label[j];
					k++;
				}
				j++;
			}

			if(Label[j]=='\0')
				continue; // Misformat! No _ found! Bail!

			k=0;
			j++; // skip the _. Now, copy out isdf stuff
			ISDFStuff[k]=Label[j]; // Compiler messup test?
			while((Label[j] != '_') && (Label[j] != '\0')) {
				if(k<MaxODFLen) {
					ISDFStuff[k]=Label[j];
					k++;
				}
				j++;
			}

			// For the CPU?
			if(strncmp(Label,"mpic",4)==0) {
				if (mySide==1)  // CPU is Scion
					BuildObject(ScionStuff,comp_team,Label);
				else
					BuildObject(ISDFStuff,comp_team,Label);
			}
			else if(strncmp(Label,"mpih",4)==0) { // For the human side
				Handle h;
				if (HumanSide==1)  // Humans are Scion
					h=BuildObject(ScionStuff,strat_team,Label);
				else
					h=BuildObject(ISDFStuff,strat_team,Label);
				SetGroup(h,1);
			}
		}
	}
}


void instantMission::SetCPUAIPlan(const int CPUSide,const int HumanSide,AIPType Type)
{
	if(!CheckedSVar3) {
		CheckedSVar3=true;
		const char *Contents=GetVarItemStr("network.session.svar3");
		if(Contents[0]== '\0') 
			UseCustomAIPs=false;
		else {
			UseCustomAIPs=true;
			strncpy(CustomAIPNameBase,Contents,254);
		}
	}

	if((Type<0) || (Type >=MAX_AIP_TYPE))
		Type=AIPType3; // Default

	char AIPFile[512];
	unsigned long EOS; // End of string

	// Prezap contents of string, so that we can safely append
	// characters w/o worrying about having to re-terminate
	memset(AIPFile,0x00,512);

	if(!UseCustomAIPs) {
		if(CPUSide==1) { // Scion
			strcpy(AIPFile,"scioninst");
			EOS=strlen(AIPFile);
			AIPFile[EOS]=AIPTypeExtensions[Type];

			if(HumanSide==1) { // vs Scion
				EOS=strlen(AIPFile);
				AIPFile[EOS]='_';
			} 
			else {
			} // No further name mangling needed.
		} // CPU is Scion
		else { // CPU is ISDF
			strcpy(AIPFile,"isdfinst");
			EOS=strlen(AIPFile);
			AIPFile[EOS]=AIPTypeExtensions[Type];
			
			if(HumanSide==2) { // vs ISDF
				EOS=strlen(AIPFile);
				AIPFile[EOS]='_';
			} 
			else {
			} // No further name mangling needed.
		} // CPU is ISDF case
	} // !UseCustomAIPs
	else {
		// UseCustomAIPs
		strcpy(AIPFile,CustomAIPNameBase);

		EOS=strlen(AIPFile);
		if(CPUSide==1) // CPU Scion
			AIPFile[EOS]='f';
		else 
			AIPFile[EOS]='i';

		if(HumanSide==1) // vs Human Scion
			AIPFile[EOS+1]='f';
		else 
			AIPFile[EOS+1]='i';

		AIPFile[EOS+2]=AIPTypeExtensions[Type];
	}

	// Always tack on file extension
	strcat(AIPFile,".aip");
	SetPlan(AIPFile,comp_team);
}


void instantMission::Setup(void)
{
	/*
		Here's where you set the values at the start.  
	*/
	start_done=false;
	time_count=1;
	inspect_bunker=false;
	second_objective=false;
	post_bunker=1;
	enemy_reinforcement_time=2000;
	friend_reinforcement_time=2500;
	end_counter=0;
	comp_team=6;
	strat_team=0;  // the default
	obj1=(-1);

	turn_counter=0;

	
	siege_on=false;
	anti_assault=false;
	siege_counter=0;
	assault_counter=false;
	late_game=false;
	have_armory=false;
	
	powerup1=NULL;
	powerup2=NULL;
	powerup3=NULL;
	powerup4=NULL;

	for (int temp=0;temp<40;temp++)
	{
		goodies[temp]=NULL;
		picking_up[temp]=false;
		captured[temp]=false;
		tuglist[temp]=NULL;
		holding[temp]=NULL;
	}

	myGoal=2;
	NumHumans=CountPlayers();
}

void instantMission::AddObject(Handle h)
{

	if (turn_counter == 0) {
		// monitor what objects get built by the mpstrat dll
		if (GetTeamNum(h) == 1) {
			if (IsOdf(h, "ivrecy_m")) {
				HumanSide=2;
				mySide = 1;
				recycler = h;
			}
			else if (IsOdf(h, "fvrecy_m")) {
				mySide = 2;
				HumanSide=1;
				recycler = h;
			}

			int ShellRace=GetVarItemInt("network.session.ivar13");
			if(!ShellRace)
				mySide=2; // ISDF
			else
				mySide=1;
		}
	}

	if (GetTeamNum(h)==comp_team)
	{
#if 0 //def _DEBUG
		SetCurHealth(h,1);
		SetMaxHealth(h,1);
		SetCurAmmo(h,1);
		SetMaxAmmo(h,1);
#endif

		SetSkill(h,difficulty+1);
		/*
		if (//(!IsAlive(h)) && 
			((IsOdf(h,"fvtank")) || (IsOdf(h,"fvarch"))))
		{
			for (int temp=0;temp<MAX_GOODIES;temp++)
			{
				if (goodies[temp]==NULL)
				{
					goodies[temp]=h;
					break;
				}
			}
		}
		*/
		if ((IsOdf(h,"ibarmo")) || (IsOdf(h,"fbstro"))) {
			have_armory=true;
		}
		if (have_armory) {
			if (IsOdf(h,"ivtank")) {
				GiveWeapon(h,"gspstab_c");
			}
			if (IsOdf(h,"fvtank")) {
				GiveWeapon(h,"garc_c");   // what a crutch!  
				GiveWeapon(h,"gshield");
			}
		}
	}
	else
	if ((myGoal==0) && (GetTeamNum(h)==strat_team))
	{

		// if it's artillery bomber/ go to late game tactics
		if ((IsOdf(h,"fvartl")) || (IsOdf(h,"ivbomb"))) {
			if ((!siege_on) && (!late_game)) {
				late_game=true;
				SetCPUAIPlan(mySide,HumanSide,AIPTypeL);
			}
		}

		if ((IsOdf(h,"ivscout")) 
			|| (IsOdf(h,"ivmisl"))
			|| (IsOdf(h,"ivtank"))
			|| (IsOdf(h,"ivrckt"))
			|| (IsOdf(h,"ivatank"))
			|| (IsOdf(h,"ivwalk"))
			|| (IsOdf(h,"ivserv"))
			|| (IsOdf(h,"fvscout")) 
			|| (IsOdf(h,"fvsent"))
			|| (IsOdf(h,"fvtank"))
			|| (IsOdf(h,"fvarch"))
			|| (IsOdf(h,"fvatank"))
			|| (IsOdf(h,"fvwalk"))
			|| (IsOdf(h,"fvserv"))
		)
		{
			SetSkill(h,3-difficulty);
			SetTeamNum(h,1);
//			int grp=GetFirstEmptyGroup();			
//			SetGroup(h,grp);
		}
	}
	if (GetTeamNum(h)==strat_team) {
	
		if ((IsOdf(h,"ibrecy_m")) || (IsOdf(h,"fbrecy_m")))
		{
			/*
				SetAip here based on turn_counter
			*/

			int stratchoice = turn_counter%2;
			if (mySide==1) {
				switch (stratchoice) {
					case 0:
#ifdef TEST_INSTANT
						SetObjectiveOn(h);
						SetObjectiveName(h,"scioninst1");
#endif
						SetCPUAIPlan(mySide,HumanSide,AIPType1);
						break;
					case 1:
#ifdef TEST_INSTANT
						SetObjectiveOn(h);
						SetObjectiveName(h,"scioninst3");
#endif
						SetCPUAIPlan(mySide,HumanSide,AIPType3);
						break;
					case 2:
#ifdef TEST_INSTANT
						SetObjectiveOn(h);
						SetObjectiveName(h,"scioninst2");
#endif
						SetCPUAIPlan(mySide,HumanSide,AIPType2);
						break;
				}
			}
			else {
				switch (stratchoice) {
					case 0:
#ifdef TEST_INSTANT
						SetObjectiveOn(h);
						SetObjectiveName(h,"isdfinst1");
#endif
						SetCPUAIPlan(mySide,HumanSide,AIPType1);
						break;
					case 1:
#ifdef TEST_INSTANT
						SetObjectiveOn(h);
						SetObjectiveName(h,"isdfinst3");
#endif
						SetCPUAIPlan(mySide,HumanSide,AIPType3);
						break;
				}
	
			}
		} else {
			if ((IsOdf(h,"ivatank")) ||  (IsOdf(h,"fvwalk"))) {
				assault_counter++;
			} 
		}
	}
		
}

void instantMission::DeleteObject(Handle h) {
	if ((IsOdf(h,"ivatank")) ||  (IsOdf(h,"fvwalk"))) {
		--assault_counter;
	}
}

void instantMission::CreateObjectives()
{
}

void instantMission::TestObjectives()
{
	// game end conditions will be handled by the strat code
}

void instantMission::Execute(bool *TeamIsSetUp, Handle *RecyclerHandles)
{
	player=GetPlayerHandle();
	turn_counter++;
	DoGenericStrategy(TeamIsSetUp, RecyclerHandles);
}



int Rand(void)
{
	return (int)GetRandomFloat(32768);
}

void instantMission::DoGenericStrategy(bool *TeamIsSetUp, Handle *RecyclerHandles)
{
//#ifdef BLAHXBLAH
//	int a;
 	time_count++;
	if (!start_done)
	{
		strat_team=1;

		start_done=true;

		SetupExtraVehicles();
		
		// get Instant My Side
		if (mySide==1) { // CPU is Scion
		//	recycler=BuildObject("ivrecy",strat_team,"Recycler");
			enemy_recycler=BuildObject("fvrecy",comp_team,"RecyclerEnemy");
			TeamIsSetUp[comp_team]=true;
			RecyclerHandles[comp_team]=enemy_recycler;

			Handle h=BuildObject("fvtank",strat_team,"Tank1");
			int grp=GetFirstEmptyGroup();			
			SetGroup(h,grp);
			h=BuildObject("fvtank",strat_team,"Tank2");
			grp=GetFirstEmptyGroup();			
			SetGroup(h,grp);

			h=BuildObject("fvtank",strat_team,"Tank3");
			grp=GetFirstEmptyGroup();			
			SetGroup(h,grp);
			BuildObject("fvturr",strat_team,"Turret1");
			BuildObject("fvturr",strat_team,"Turret2");
			BuildObject("fvturr",strat_team,"Turret3");

			BuildObject("fvturr",comp_team,"turretEnemy1");
			BuildObject("fvturr",comp_team,"turretEnemy2");
			if (compForce>0)
			{
				BuildObject("fvturr",comp_team,"gtow2");
				BuildObject("fvturr",comp_team,"gtow3");
			}
			if (compForce>1)
			{
				BuildObject("fvturr",comp_team,"gtow4");
				BuildObject("fvturr",comp_team,"gtow5");
				BuildObject("fvtank",comp_team,"tankEnemy1");
				BuildObject("fvtank",comp_team,"tankEnemy2");
				BuildObject("fvtank",comp_team,"tankEnemy3");
			}
			if (myForce>1)
			{
				h=BuildObject("ivtank",strat_team,"tank1");
				grp=GetFirstEmptyGroup();			
				SetGroup(h,grp+1);
				h=BuildObject("ivtank",strat_team,"tank2");
				grp=GetFirstEmptyGroup();			
				SetGroup(h,grp+1);
				h=BuildObject("ivtank",strat_team,"tank3");
				grp=GetFirstEmptyGroup();			
				SetGroup(h,grp+1);

			}


		}
		else {
		//	recycler=BuildObject("fvrecy",strat_team,"Recycler");
			enemy_recycler=BuildObject("ivrecy",comp_team,"RecyclerEnemy");
			TeamIsSetUp[comp_team]=true;
			RecyclerHandles[comp_team]=enemy_recycler;

			Handle h=BuildObject("ivtank",strat_team,"Tank1");
			int grp=GetFirstEmptyGroup();			
			SetGroup(h,grp);
			h=BuildObject("ivtank",strat_team,"Tank2");
			grp=GetFirstEmptyGroup();			
			SetGroup(h,grp);
			h=BuildObject("ivtank",strat_team,"Tank3");
			grp=GetFirstEmptyGroup();			
			SetGroup(h,grp);
			BuildObject("ivturr",strat_team,"Turret1");
			BuildObject("ivturr",strat_team,"Turret2");
			BuildObject("ivturr",strat_team,"Turret3");

			BuildObject("ivturr",comp_team,"turretEnemy1");
			BuildObject("ivturr",comp_team,"turretEnemy2");
			if (compForce>0)
			{
				BuildObject("ivturr",comp_team,"gtow2");
				BuildObject("ivturr",comp_team,"gtow3");
			}
			if (compForce>1)
			{
				BuildObject("ivturr",comp_team,"gtow4");
				BuildObject("ivturr",comp_team,"gtow5");
				BuildObject("ivtank",comp_team,"tankEnemy1");
				BuildObject("ivtank",comp_team,"tankEnemy2");
				BuildObject("ivtank",comp_team,"tankEnemy3");
			}
			if (myForce>1)
			{
				h=BuildObject("ivtank",strat_team,"tank1");
				grp=GetFirstEmptyGroup();			
				SetGroup(h,grp+1);
				h=BuildObject("ivtank",strat_team,"tank2");
				grp=GetFirstEmptyGroup();			
				SetGroup(h,grp+1);
				h=BuildObject("ivtank",strat_team,"tank3");
				grp=GetFirstEmptyGroup();			
				SetGroup(h,grp+1);
			}
		}
		int grp=GetFirstEmptyGroup();			
		SetGroup(recycler,grp);
		if (myGoal==0)
		{
			// check my team..
			if (HumanSide==2) {
				SetPlan("isdfteam.aip",strat_team);
			}
			else
			{
				SetPlan("scionteam.aip",strat_team);
			}	
		}

		SetCPUAIPlan(mySide,HumanSide,AIPType0);
		// do enemies
		//					ClearObjectives();
		//					AddObjective("isdf0501.otf",WHITE,5.0f);

		// because turrets suck
//		BuildObject("fvscout",comp_team,"turretEnemy1");
		if (compForce>0)
		{
			if (mySide==1) { // CPU is Scion
				BuildObject("fvtank",comp_team,"TankEnemy1");
				BuildObject("fvtank",comp_team,"TankEnemy2");
				BuildObject("fvsent",comp_team,"SentryEnemy1");
				BuildObject("fvsent",comp_team,"SentryEnemy2");
				if (compForce>1) {
					BuildObject("fvtank",comp_team,"TankEnemy3");
					BuildObject("fvsent",comp_team,"SentryEnemy3");	
				}
			}
			else { // CPU is ISDF
				BuildObject("ivtank",comp_team,"TankEnemy1");
				BuildObject("ivtank",comp_team,"TankEnemy2");
				BuildObject("ivscout",comp_team,"SentryEnemy1");
				BuildObject("ivscout",comp_team,"SentryEnemy2");
				if (compForce>1) {
					BuildObject("ivtank",comp_team,"TankEnemy3");
					BuildObject("ivscout",comp_team,"SentryEnemy3");	
				}
			}
		}

		//		BuildObject("ivscout",strat_team,"Scout1");
		//		BuildObject("ivscout",strat_team,"Scout2");
		//		BuildObject("ivscout",strat_team,"Scout3");
		//		BuildObject("ivscav",strat_team,"Scavenger");

		if (mySide==1) // CPU is Scion
			BuildObject("fvscav",comp_team,"ScavengerEnemy");
		else
			BuildObject("ivscav",comp_team,"ScavengerEnemy");

		// power ups?
		BuildObject("apammo",0,"ammo1");
		BuildObject("apammo",0,"ammo2");
		BuildObject("apammo",0,"ammo3");
		BuildObject("aprepa",0,"repair1");
		BuildObject("aprepa",0,"repair2");
		BuildObject("aprepa",0,"repair3");
		// objectives .. later make this optional	
//		CreateObjectives();
		SetScrap(comp_team,60);
		SetScrap(1,40);
	}	
	if (time_count%enemy_reinforcement_time==0)
	{
//		AddScrap(comp_team,5);
		if (compForce>0)
		{
//			BuildObject("fvtank",comp_team,"TankEnemy1");
//			BuildObject("fvtank",comp_team,"TankEnemy2");
//			BuildObject("fvsent",comp_team,"SentryEnemy1");
//			BuildObject("fvsent",comp_team,"SentryEnemy2");
//			AddScrap(comp_team,10);
			if (compForce>1)
			{
//				BuildObject("fvtank",comp_team,"TankEnemy3");
//				BuildObject("fvsent",comp_team,"SentryEnemy3");	
				AddScrap(comp_team,10);
			}
		}
	}

	if ((time_count%100)==0) { // Evaluate this every 10 seconds
		NumHumans=CountPlayers();
	}

	if (time_count%(30-(compForce*10)-NumHumans+1)==0)  // this speeds up resources
		AddScrap(comp_team,1);
	//	if (time_count%(100-(myForce*20))==0)
	//		AddScrap(strat_team,1);
	if (time_count%friend_reinforcement_time==0)
	{
/*
		BuildObject("ivtank",strat_team,"Tank1");
		BuildObject("ivtank",strat_team,"Tank2");
		BuildObject("ivtank",strat_team,"Tank3");
		BuildObject("ivturr",strat_team,"Turret1");
		BuildObject("ivturr",strat_team,"Turret2");
		BuildObject("ivturr",strat_team,"Turret3");
		BuildObject("ivscout",strat_team,"Scout1");
		BuildObject("ivscout",strat_team,"Scout2");
		BuildObject("ivscout",strat_team,"Scout3");
*/
		// power ups?
		BuildObject("apammo",0,"ammo1");
		BuildObject("apammo",0,"ammo2");
		BuildObject("apammo",0,"ammo3");
		BuildObject("aprepa",0,"repair1");
		BuildObject("aprepa",0,"repair2");
		BuildObject("aprepa",0,"repair3");

	}


	if (myGoal==0) {
		powerup_counter++;
		if (powerup_counter%200==0) {
			if (!IsAround(powerup1)) {
				powerup1=BuildObject("apshdw",0,"power1");
			}
			if (!IsAround(powerup2)) {
				powerup2=BuildObject("apsstb",0,"power2");
			}
			if (!IsAround(powerup3)) {
				powerup3=BuildObject("apffld",0,"power3");
			}
			if (!IsAround(powerup4)) {
				powerup4=BuildObject("apspln",0,"power4");
			}
		}
	}

	// AIP switch logic
	if (!siege_on) {
		Handle enemy=GetNearestEnemy(enemy_recycler);
		if (GetDistance(enemy,enemy_recycler)<300.0f) { // under siege
			siege_counter++;
		} else {
			siege_counter=0;
		}
		if (siege_counter>450) { // 45 sec, if we don't change the sim rate :-)
			siege_on=true;

			SetCPUAIPlan(mySide,HumanSide,AIPTypeS);

		}
	}  // !siege_on
	else {
		Handle enemy=GetNearestEnemy(enemy_recycler);
		if (GetDistance(enemy,enemy_recycler)>301.0f) {
			//			SetPlan("scioninstl.aip",comp_team);
			siege_on=false;

			SetCPUAIPlan(mySide,HumanSide,AIPTypeL);
		}
	}  // siege_on
	// assault_test
	if ((!late_game) && (!siege_on) && (!anti_assault) && (assault_counter>2)) {
		anti_assault=true;

		SetCPUAIPlan(mySide,HumanSide,AIPTypeA);
	} else {
		if ((anti_assault) && (assault_counter<3)) {
			anti_assault=false;
			SetCPUAIPlan(mySide,HumanSide,AIPTypeL);
		}
	}

	TestObjectives();
//#endif
}

EjectKillRetCodes instantMission::PlayerEjected(void)
{
	return DoEjectPilot;

}


EjectKillRetCodes instantMission::PlayerKilled(int KillersHandle)
{
	return DoEjectPilot;  // Game over, man.
}

SubMission * BuildSubMission(void)
{
	return new instantMission();
}

